package org.tonylin.util.exec;
import java.util.concurrent.Callable;

import org.tonylin.util.ThreadUtil;


public class RetryableTask<T> implements IRetryableTask<T> {

	private Callable<T> mCallable = null;
	private Object mResult = null;
	private ISleepStrategy mSleepStrategy = new BasicSleepStrategy();
	private IRetryablePolicy mRetryablePolicy = new ExceptionRetryablePolicy();
	
	public RetryableTask(Callable<T> callable){
		mCallable = callable;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public T call() throws Throwable {
		boolean isFirstTime = true;
		do {
			if(!isFirstTime && mSleepStrategy != null){
				ThreadUtil.sleep(mSleepStrategy.getSleepTime());
			}
			try {
				T result = mCallable.call();
				mResult = result;
			} catch( Exception e ){
				mResult = e;
			}
		} while( isNeedToRetry(mResult) );
		
		if( mResult instanceof Throwable ){
			throw (Throwable)mResult;
		}
		return (T)mResult;
	}

	@Override
	public void setSleepStrategy(ISleepStrategy aStrategy) {
		mSleepStrategy = aStrategy;
	}
	
	@Override
	public void setRetryablePolicy(IRetryablePolicy policy){
		mRetryablePolicy = policy;
	}
	
	private boolean isNeedToRetry(Object aData){
		if( mRetryablePolicy != null ){
			return mRetryablePolicy.needToRetry(aData);
		}
		return false;
	}
}
